#!/usr/bin/env python
# -*- coding: utf-8 -*-

# © 2017-2019, ETH Zurich, Institut für Theoretische Physik
# Author: Dominik Gresch <greschd@gmx.ch>

try:
    from collections import ChainMap
except ImportError:
    from chainmap import ChainMap



# -*- coding: utf-8 -*-

# © 2017-2019, ETH Zurich, Institut für Theoretische Physik
# Author: Dominik Gresch <greschd@gmx.ch>

import time

import pytest
from fsc.export import export

@export
@pytest.fixture
def get_process_builder(inputs_setup):
    def inner(calculation_string, code_string, single_core=True):
        from aiida.plugins import CalculationFactory
        builder = CalculationFactory(calculation_string).get_builder()
        inputs_setup(
            builder,
            code_string=code_string,
            single_core=single_core
        )
        return builder
    return inner

@export
@pytest.fixture
def inputs_setup(set_code, set_single_core):
    def inner(builder, code_string, single_core=True):
        set_code(builder, code_string=code_string)
        if single_core:
            set_single_core(builder)
    return inner

@export
@pytest.fixture
def set_code():
    def inner(builder, code_string):
        from aiida.orm import Code
        builder.code = Code.get_from_string(code_string)
    return inner

@export
@pytest.fixture
def set_single_core():
    def inner(builder):
        builder.metadata.options = dict(
            resources={'num_machines': 1, 'tot_num_mpiprocs': 1},
            withmpi=False
        )
    return inner

# TODO: Check for uses of these fixtures, and if / how they need to be changed.

@export
@pytest.fixture
def assert_state():
    def inner(pid, state):
        from aiida.orm import load_node
        from aiida.engine import ProcessState
        if isinstance(state, ProcessState):
            state = state.value
        calc_node = load_node(pid)
        assert calc_node.get_attribute('process_state') == state
    return inner

@export
@pytest.fixture
def assert_finished(assert_state):
    def inner(pid):
        from aiida.engine import ProcessState
        assert_state(pid, ProcessState.FINISHED)
    return inner

@export
@pytest.fixture
def wait_for():
    def inner(pid, timeout=1):
        from aiida.orm import load_node
        calc = load_node(pid)
        while not calc.is_terminated:
            time.sleep(timeout)
    return inner



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# © 2017-2019, ETH Zurich, Institut für Theoretische Physik
# Author: Dominik Gresch <greschd@gmx.ch>

import os

from aiida.cmdline.params.types import ComputerParamType, PluginParamType
from aiida.cmdline.commands.cmd_code import setup_code as _setup_code

from .contextmanagers import redirect_stdout

def setup_code(
    label,
    description,
    default_plugin,
    remote_computer,
    remote_abspath,
    local=False,
    prepend_text='',
    append_text=''
):
    with open(os.devnull, 'w') as devnull, redirect_stdout(devnull):
        _setup_code.callback(
            label=label,
            description=description,
            input_plugin=PluginParamType(group='calculations')(default_plugin),
            computer=ComputerParamType()(remote_computer),
            remote_abs_path=remote_abspath,
            on_computer=not local,
            prepend_text=prepend_text,
            append_text=append_text,
            non_interactive=True,
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# © 2017-2019, ETH Zurich, Institut für Theoretische Physik
# Author: Dominik Gresch <greschd@gmx.ch>

import sys
import contextlib

@contextlib.contextmanager
def redirect_stdout(target):
    original = sys.stdout
    sys.stdout = target
    yield
    sys.stdout = original



# -*- coding: utf-8 -*-

# © 2017-2019, ETH Zurich, Institut für Theoretische Physik
# Author: Dominik Gresch <greschd@gmx.ch>
"""
Defines a fixture to check if the AiiDA entrypoints (workflows, calculations,
parsers, data, transports) are loadable through the corresponding factory.
"""

import pkg_resources

import pytest
from fsc.export import export


@export
@pytest.fixture
def check_entrypoints(configure):  # pylint: disable=unused-argument
    """
    Fixture to check that loading of all the workflow, calculation and parser
    entrypoints through the corresponding factory works for the given (base)
    module name.
    """
    def inner(module_name):  # pylint: disable=missing-docstring
        from aiida.plugins.factories import WorkflowFactory, CalculationFactory, DataFactory, ParserFactory, TransportFactory
        for entrypoint_name, factory in [
            ('aiida.workflows', WorkflowFactory),
            ('aiida.calculations', CalculationFactory),
            ('aiida.parsers', ParserFactory), ('aiida.data', DataFactory),
            ('aiida.transports', TransportFactory)
        ]:
            for entry_point in pkg_resources.iter_entry_points(
                entrypoint_name
            ):
                if entry_point.module_name.split('.')[0] == module_name:
                    factory(entry_point.name)

    return inner



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# © 2017-2019, ETH Zurich, Institut für Theoretische Physik
# Author: Dominik Gresch <greschd@gmx.ch>

import inspect
try:
    import pathlib
except ImportError:
    import pathlib2 as pathlib
from os.path import abspath, join, dirname

import pytest
from fsc.export import export

@export
@pytest.fixture
def sample(request):
    def inner(name):
        samples_dir = join(dirname(abspath(str(request.fspath))), 'samples')
        return join(samples_dir, name)
    return inner



# -*- coding: utf-8 -*-

# © 2017-2019, ETH Zurich, Institut für Theoretische Physik
# Author: Dominik Gresch <greschd@gmx.ch>

from __future__ import division, print_function

import os
import copy
import shutil
import subprocess32 as subprocess
from builtins import input
from contextlib import contextmanager

import yaml

import aiida
from aiida.manage.tests.pytest_fixtures import aiida_profile

import pytest
from fsc.export import export


__all__ = ['aiida_profile'] # Need to export the fixtures that we depend on.

@export
def pytest_addoption(parser):
    parser.addoption(
        '--queue-name',
        action='store',
        help='Name of the queue used to submit calculations.'
    )
    parser.addoption(
        '--quiet-wipe',
        action='store_true',
        help='Disable asking for input before wiping the test AiiDA environment.'
    )
    parser.addoption(
        '--print-status',
        action='store_true',
        help='Print the calculation and work status before exiting.'
    )
    parser.addoption(
        '--end-cmd',
        action='store',
        help=
        'Command to run before the tear-down (with output capture suspended).'
    )


@export
@pytest.fixture(scope='session')
def config_dict():
    with open(os.path.abspath('config.yml'), 'r') as f:
        config = yaml.load(f)
    if config is None:
        config = dict()
    return config


@export
@pytest.fixture(scope='session')
def get_queue_name_from_code(request, config_dict):
    def inner(code):
        queue_name = request.config.getoption('--queue-name')
        if queue_name is None:
            computer = config_dict['codes'][code]['remote_computer']
            queue_name = config_dict['computers'][computer]['queue_name']
        return queue_name

    return inner


@export
@pytest.fixture(scope='session')
def configure_with_daemon(configure):
    subprocess.run(['verdi', 'daemon', 'start'],
                   env=os.environ,
                   stdout=subprocess.DEVNULL)
    yield
    subprocess.run(['verdi', 'daemon', 'stop'],
                   env=os.environ,
                   stdout=subprocess.DEVNULL)


@export
@pytest.fixture(scope='session')
def configure(pytestconfig, config_dict, aiida_profile):
    config = copy.deepcopy(config_dict)
    os.environ['AIIDA_PATH'] = aiida_profile._manager.root_dir

    from ._computer import setup_computer
    computers = config.get('computers', {})
    for name, kwargs in computers.items():
        setup_computer(
            name=name,
            **{k: v
                for k, v in kwargs.items() if k != 'queue_name'}
        )

    from ._code import setup_code
    codes = config.get('codes', {})
    for label, kwargs in codes.items():
        setup_code(label=label, **kwargs)

    # with same pattern setup test psf- pseudo family
    from ._pseudo_family import setup_pseudo_family
    pseudo_families = config.get('pseudo_families', {})
    for group_name, kwargs in pseudo_families.items():
        setup_pseudo_family(group_name=group_name, **kwargs)

    # aiida.load_profile()
    yield

    # Handle compatibility break in pytest
    capture_manager = pytestconfig.pluginmanager.getplugin(
        'capturemanager'
    )
    init = getattr(
        capture_manager, 'init_capturings',
        getattr(capture_manager, 'start_global_capturing', None)
    )
    suspend = getattr(
        capture_manager, 'suspendcapture',
        getattr(capture_manager, 'suspend_global_capture', None)
    )
    resume = getattr(
        capture_manager, 'resumecapture',
        getattr(capture_manager, 'resume_global_capture', None)
    )

    @contextmanager
    def suspend_capture():
        try:
            init()
        except AssertionError:
            pass
        suspend(in_=True)
        yield
        resume()

    if pytestconfig.option.print_status:
        with suspend_capture():
            print('\n\nProcess List:')
            subprocess.call(['verdi', 'process', 'list', '-a'])
    end_cmd = pytestconfig.option.end_cmd
    if end_cmd is not None:
        with suspend_capture():
            print("Executing '{}'".format(end_cmd))
            subprocess.call(end_cmd, shell=True)

    if not pytestconfig.option.quiet_wipe:
        with suspend_capture():
            print('\nAiiDA root directory: {}'.format(aiida_profile._manager.root_dir))
            input(
                "Tests finished. Press enter to wipe the test AiiDA environment."
            )


@contextmanager
def reset_after_run():
    config_folder = os.path.expanduser(
        aiida.manage.configuration.settings.AIIDA_CONFIG_FOLDER
    )
    config_save_folder = os.path.join(
        os.path.dirname(config_folder), '.aiida~'
    )
    reset_config(config_folder, config_save_folder)
    shutil.copytree(config_folder, config_save_folder)
    try:
        yield
    except Exception as e:
        raise e
    finally:
        reset_config(config_folder, config_save_folder)
        reset_submit_test_folder(config_folder)


def reset_config(config_folder, config_save_folder):
    if os.path.isdir(config_save_folder):
        shutil.rmtree(config_folder, ignore_errors=True)
        os.rename(config_save_folder, config_folder)


def reset_submit_test_folder(config_folder):
    submit_test_folder = os.path.join(
        os.path.dirname(config_folder), 'submit_test'
    )
    if os.path.isdir(submit_test_folder):
        # remove temp `submit_test` folder for not_submitted_to_daemon tests
        shutil.rmtree(submit_test_folder)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# © 2017-2019, ETH Zurich, Institut für Theoretische Physik
# Author: Dominik Gresch <greschd@gmx.ch>

__version__ = '0.1.0a6'

from ._process import *
from ._configure import *
from ._sample import *
from ._compare import *
from ._entrypoints import *

__all__ = _process.__all__ + _configure.__all__ + _sample.__all__ + _compare.__all__ + _entrypoints.__all__



# -*- coding: utf-8 -*-

# © 2017-2019, ETH Zurich, Institut für Theoretische Physik
# Author: Dominik Gresch <greschd@gmx.ch>

import pytest
from fsc.export import export

@export
@pytest.fixture
def assert_outputs_equal():
    def inner(output1, output2):
        def normalize_output(output):
            return {key: value.uuid for key, value in output.items()}
        assert normalize_output(output1) == normalize_output(output2)
    return inner



# -*- coding: utf-8 -*-

# © 2017-2019, ETH Zurich, Institut für Theoretische Physik
# Author: Dominik Gresch <greschd@gmx.ch>

import os
import re
import sys
from setuptools import setup, find_packages

# Get the version number
with open('./aiida_pytest/__init__.py') as f:
    match_expr = "__version__[^'\"]+(['\"])([^'\"]+)"
    version = re.search(match_expr, f.read()).group(2).strip()

README = 'A module to simplify using pytest for AiiDA plugins.'

if __name__ == '__main__':
    setup(
        name='aiida-pytest',
        version=version,
        description=README,
        long_description=open(
            os.path.join(
                os.path.dirname(os.path.abspath(__file__)), 'README.md'
            )
        ).read(),
        long_description_content_type="text/markdown",
        readme=README,
        author='Dominik Gresch',
        author_email='greschd@gmx.ch',
        license='Apache 2.0',
        classifiers=[
            'Development Status :: 3 - Alpha',
            'Environment :: Plugins',
            'Intended Audience :: Science/Research',
            'License :: OSI Approved :: Apache Software License',
            'Programming Language :: Python :: 2.7',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.6',
            'Programming Language :: Python :: 3.7',
            'Topic :: Scientific/Engineering :: Physics',
            'Topic :: Software Development :: Testing',
            'Framework :: Pytest',
            'Framework :: AiiDA',
        ],
        keywords='pytest aiida workflows',
        packages=find_packages(exclude=['aiida', 'plum']),
        include_package_data=True,
        install_requires=[
            'aiida-core>=1.0.0,<2.0.0', 'pytest', 'pyyaml', 'fsc.export',
            'pgtest>=1.1', 'future', 'subprocess32'
        ],
        extras_require={
            ':python_version < "3"': ['chainmap', 'pathlib2']
        }
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# © 2017-2019, ETH Zurich, Institut für Theoretische Physik
# Author: Dominik Gresch <greschd@gmx.ch>

import os

from aiida.common.utils import get_object_from_string

from .contextmanagers import redirect_stdout


def setup_pseudo_family(command_name, folder, group_name, group_description):
    pseudo_uploadfamily_cmd = get_object_from_string(command_name)
    with open(os.devnull, 'w') as devnull, redirect_stdout(devnull):
        pseudo_uploadfamily_cmd.callback(
            path=folder,
            name=group_name,
            description=group_description,
            stop_if_existing=False,
            dry_run=False)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# © 2017-2019, ETH Zurich, Institut für Theoretische Physik
# Author: Dominik Gresch <greschd@gmx.ch>

import os
import copy
import getpass

import click
from aiida.cmdline.params.types import PluginParamType, UserParamType, ComputerParamType
from aiida.cmdline.commands.cmd_computer import (
    verdi_computer, computer_setup as _setup_computer, computer_configure as
    _configure_computer
)

from .contextmanagers import redirect_stdout


def setup_computer(
    name,
    hostname,
    transport,
    scheduler,
    work_directory,
    shebang='#!/bin/bash',
    configuration={},
    description='',
    mpirun_command='mpirun -np {tot_num_mpiprocs}',
    num_cpus=1,
    prepend_text=None,
    append_text=None
):
    configuration = copy.deepcopy(configuration)
    if transport == 'ssh':
        configuration.setdefault('username', getpass.getuser())
        configuration.setdefault('look_for_keys', True)
        configuration.setdefault('allow_agent', True)
        configuration.setdefault('load_system_host_keys', True)
    with open(os.devnull, 'w') as devnull, redirect_stdout(devnull):
        click.Context(verdi_computer).invoke(
            _setup_computer,
            label=name,
            hostname=hostname,
            transport=PluginParamType(group='transports')(transport),
            scheduler=PluginParamType(group='schedulers')(scheduler),
            description=description,
            work_dir=work_directory,
            shebang=shebang,
            mpirun_command=mpirun_command,
            mpiprocs_per_machine=num_cpus,
            prepend_text=prepend_text or '',
            append_text=append_text or '',
            non_interactive=True,
        )
    configure_command = _configure_computer.commands[transport]
    with open(os.devnull, 'w') as devnull, redirect_stdout(devnull):
        configure_command.callback(
            computer=ComputerParamType()(name),
            user=UserParamType()('tests@aiida.mail'),
            non_interactive=True,
            **configuration
        )
